function B2_shape
% B2_shape illustrates the shape of the first and second level of the bias function induced by max(x1,x2), 
% where x1 and x2 are two normal variates, as discussed in Qian (2011)
% (x1,x2) ~ MVN[(mu1,mu2),(sigma11,sigma12;sigma21, sigma22)]
% parameter of interest: max(mu1,mu2)
% naive estimator: max(x1,x2)
sigma11 = 1;
sigma22 = 1;
sigma12 = 0;
sigma0 = sqrt(sigma11 + sigma22 - 2 * sigma12);
P = chol([sigma11, sigma12;sigma12, sigma22]);


mu1 = 0;
mu2 = -5:0.1:5;
ngrid = length(mu2);
nsim = 50000;
B2 = zeros(1, ngrid);

H_status_bar = waitbar(0);
for m = 1:ngrid
    waitbar(m / ngrid, H_status_bar);
    mu2_use = mu2(m);
    mu_sim = repmat([mu1;mu2_use], 1, nsim) + P' * randn(2,nsim);
    B2(m) =  B1(mu1, mu2_use, sigma0) - mean( B1(mu_sim(1,:), mu_sim(2,:), sigma0) ) ;     
end
close(H_status_bar);


figure(2)
plot(mu2,B1(mu1(:, ones(1,ngrid)), mu2, sigma0),'-b',mu2,B2,'--r');
title('The shape of the bias functions B_1 and B_2');
xlabel('\mu_2');
ylabel('Bias');
legend('B1 curve', 'B2 curve','location','Northeast');

% Plot a 3-D graph of B1
mu1 = -3:0.1:3;
mu2 = -3:0.1:3;
[X, Y] = meshgrid(mu1,mu2);
Z = zeros(length(mu1),length(mu2));
Z(:) = B1(X(:)',Y(:)',sigma0);
figure(1)
mesh(mu1,mu2,Z);
title('The shape of the bias function B_1');
xlabel('\mu_1');
ylabel('\mu_2');
zlabel('Bias');

end

function val = B1(mu1, mu2, sigma0)

weight = normcdf((mu1 - mu2) / sigma0);
val = weight .* mu1 + (1 - weight) .* mu2 + sigma0 * normpdf((mu1 - mu2) / sigma0) - max([mu1; mu2]);
end



