function [lb, ub] = Example_Kreider(L, X, Y, Z, nlevel, ndraws)

mask = logical(dummyvar(Z));
[nobs, ngroup] = size(mask);

% Step 1     compute the MIV lb, ub for P(L=1 | W=1)
% Step 1.1  find lb, ub under each MIV value, and its std
syms p1 p2 p3 p4 p5 p6 p7 p8;
lb_sym = p8 / (p8 + p4 + p3 + p1);
ub_sym = (p8 + p7 + p5) / (p8 + p4 + p7 + p5);
Grad_lb_sym = jacobian(lb_sym, [p1, p2, p3, p4, p5, p6, p7, p8]);
Grad_ub_sym = jacobian(ub_sym, [p1, p2, p3, p4, p5, p6, p7, p8]);

analog_lb = zeros(1,ngroup);
analog_ub = zeros(1,ngroup);
std_lb = zeros(1,ngroup);
std_ub = zeros(1,ngroup);
for m = 1:ngroup
    L_use = L(mask(:,m));
    X_use = X(mask(:,m));
    Y_use = Y(mask(:,m));        
    ind = L_use * 4 + X_use * 2 + Y_use + 1;    
    
    p1 = sum(ind == 1) / nobs; p2 = sum(ind == 2) / nobs; p3 = sum(ind == 3) / nobs; p4 = sum(ind == 4) / nobs; 
    p5 = sum(ind == 5) / nobs; p6 = sum(ind == 6) / nobs; p7 = sum(ind == 7) / nobs; p8 = sum(ind == 8) / nobs; 
    analog_lb(m) = eval(lb_sym);
    analog_ub(m) = eval(ub_sym);
    Grad_lb = eval(Grad_lb_sym);
    Grad_ub = eval(Grad_ub_sym);
    std_lb(m) = sqrt(sum([p1 p2 p3 p4 p5 p6 p7 p8] .* Grad_lb.^ 2) / nobs);
    std_ub(m) = sqrt(sum([p1 p2 p3 p4 p5 p6 p7 p8] .* Grad_ub.^ 2) / nobs);
end

% Step 1.2  use max(.) to find the MIV lb, min(.) to find the MIV ub
H_status_bar = waitbar(0,'Simulation in progress   0%');
lb_MIV = zeros(ngroup, nlevel+1);
ub_MIV = zeros(ngroup, nlevel+1);
lb_conserve = zeros(ngroup,1);
ub_conserve = zeros(ngroup,1);
for m = 1:ngroup
    
    waitbar(m / (ngroup*2) ,H_status_bar,['Simulation in progress   ',num2str(floor(m / (ngroup*2) * 100)),'%']);

    if m == ngroup
        lb_MIV(m,:) = analog_lb(ngroup);
        lb_conserve(m) = analog_lb(ngroup);
    else
        lb_MIV(m,:) = BIAS_CORRECT_MAX(analog_lb(m:ngroup), std_lb(m:ngroup), nlevel, ndraws);
        lb_conserve(m) = max(analog_lb(m:ngroup)) - mean(max(randn(ngroup-m+1,ndraws) .* repmat(std_lb(m:ngroup)',1,ndraws)));
    end
    
    
    if m == 1
        ub_MIV(m,:) = analog_ub(1);
        ub_conserve(m) = analog_ub(1);
    else
        ub_MIV(m,:) = BIAS_CORRECT_MIN(analog_ub(1:m), std_ub(1:m), nlevel, ndraws);
        ub_conserve(m) = min(analog_ub(1:m)) - mean(min(randn(m,ndraws) .* repmat(std_ub(1:m)',1,ndraws)));
    end
    
end

% Step 1.3  marginalize and get lb,ub for P(L=1 | W=1)
prob_MIV = sum(mask) / nobs;
lb11 = prob_MIV * lb_MIV;
ub11 = prob_MIV * ub_MIV;
lb_conserve11 = prob_MIV * lb_conserve;
ub_conserve11 = prob_MIV * ub_conserve;

%------------------------------------------
% Step 2     compute the MIV lb, ub for P(L=1 | W=0)
% Step 2.1  find lb, ub under each MIV value, and its std
syms p1 p2 p3 p4 p5 p6 p7 p8;
lb_sym = p6 / (p6 + p2 + p1 + p3);
ub_sym = (p6 + p5 + p7) / (p6 + p2 + p5 + p7);
Grad_lb_sym = jacobian(lb_sym, [p1, p2, p3, p4, p5, p6, p7, p8]);
Grad_ub_sym = jacobian(ub_sym, [p1, p2, p3, p4, p5, p6, p7, p8]);

analog_lb = zeros(1,ngroup);
analog_ub = zeros(1,ngroup);
std_lb = zeros(1,ngroup);
std_ub = zeros(1,ngroup);
for m = 1:ngroup
    L_use = L(mask(:,m));
    X_use = X(mask(:,m));
    Y_use = Y(mask(:,m));        
    ind = L_use * 4 + X_use * 2 + Y_use + 1;    
    
    p1 = sum(ind == 1) / nobs; p2 = sum(ind == 2) / nobs; p3 = sum(ind == 3) / nobs; p4 = sum(ind == 4) / nobs; 
    p5 = sum(ind == 5) / nobs; p6 = sum(ind == 6) / nobs; p7 = sum(ind == 7) / nobs; p8 = sum(ind == 8) / nobs; 
    analog_lb(m) = eval(lb_sym);
    analog_ub(m) = eval(ub_sym);
    Grad_lb = eval(Grad_lb_sym);
    Grad_ub = eval(Grad_ub_sym);
    std_lb(m) = sqrt(sum([p1 p2 p3 p4 p5 p6 p7 p8] .* Grad_lb.^ 2) / nobs);
    std_ub(m) = sqrt(sum([p1 p2 p3 p4 p5 p6 p7 p8] .* Grad_ub.^ 2) / nobs);
end

% Step 2.2  use max(.) to find the MIV lb, min(.) to find the MIV ub
lb_MIV = zeros(ngroup, nlevel+1);
ub_MIV = zeros(ngroup, nlevel+1);
lb_conserve = zeros(ngroup,1);
ub_conserve = zeros(ngroup,1);
for m = 1:ngroup
    
    waitbar((m+ngroup) / (ngroup*2) ,H_status_bar,['Simulation in progress   ',num2str(floor((m+ngroup) / (ngroup*2) * 100)),'%']);
     
   if m == ngroup
        lb_MIV(m,:) = analog_lb(ngroup);
        lb_conserve(m) = analog_lb(ngroup);
    else
        lb_MIV(m,:) = BIAS_CORRECT_MAX(analog_lb(m:ngroup), std_lb(m:ngroup), nlevel, ndraws);
        lb_conserve(m) = max(analog_lb(m:ngroup)) - mean(max(randn(ngroup-m+1,ndraws) .* repmat(std_lb(m:ngroup)',1,ndraws)));
    end
    
    if m == 1
        ub_MIV(m,:) = analog_ub(1);
        ub_conserve(m) = analog_ub(1);
    else
        ub_MIV(m,:) = BIAS_CORRECT_MIN(analog_ub(1:m), std_ub(1:m), nlevel, ndraws);
        ub_conserve(m) = min(analog_ub(1:m)) - mean(min(randn(m,ndraws) .* repmat(std_ub(1:m)',1,ndraws)));
    end   
end
close(H_status_bar);

% Step 2.3  marginalize and get lb,ub for P(L=1 | W=0)
prob_MIV = sum(mask) / nobs;
lb10 = prob_MIV * lb_MIV;
ub10 = prob_MIV * ub_MIV;
lb_conserve10 = prob_MIV * lb_conserve;
ub_conserve10 = prob_MIV * ub_conserve;
    
%--------------------------------------------------------
% Step 3  find the lb, ub for P(L=1 | W=1) - P(L=1 | W=0)
lb = lb11 - ub10;
ub = ub11 - lb10;
lb_conserve = lb_conserve11 - ub_conserve10;
ub_conserve = ub_conserve11 - lb_conserve10;
lb = [lb, lb_conserve];
ub = [ub, ub_conserve];