clear all
tic

% This routine replicate the empirical studies on disablity and employment in Kreider and Pepper(2007) 
% Simultaneous multi-level bootstrap (Qian, 2011) are aslo added
% to further correct the bias.
% This rountine calls the function Kreider_example

myseed = 20110402;
try
    RandStream.setDefaultStream(RandStream('mt19937ar','seed',myseed));
catch
    randn('seed',myseed);
    rand('seed',myseed);
end

% Config parameters
nlevel = 3;
ndraws = 50000;

% Loading HRS data
data_raw = load('HRS.txt');
data = [data_raw(1:2:end,:), data_raw(2:2:end,:)];
X = data(:, 3);
L = data(:, 4);
Y1 = data(:, 5);
Y2 = data(:, 11);
Y3 = L;
Y4 = 1 - X;
Y5 = (Y1 + Y2 + Y3 + Y4) >= 1;
Z_raw = repmat(1:25, 500,1);
Z = [Z_raw(:); 25; 25; 25];

% Compute the bootstap corrected MIV bounds under 5 verification scenarios.
lb = zeros(nlevel+2,5);
ub = zeros(nlevel+2,5);
[lb(:,1), ub(:,1)] = Kreider_example(L, X, Y1, Z, nlevel, ndraws);
disp(' ')
disp(['HRS Beneficiaries:  ratio = ', num2str(mean(Y1))])
for m = 1:nlevel+1
    fprintf( 'T%1.0f   [%5.3f, %5.3f] \n', m, lb(m,1), ub(m,1));
end
fprintf( 'Tc   [%5.3f, %5.3f] \n', lb(nlevel+2,1), ub(nlevel+2,1));

[lb(:,2), ub(:,2)] = Kreider_example(L, X, Y2, Z, nlevel, ndraws);
disp(' ')
disp(['HRS Wave 2 verification:  ratio = ', num2str(mean(Y2))])
for m = 1:nlevel+1
    fprintf( 'T%1.0f   [%5.3f, %5.3f] \n', m, lb(m,2), ub(m,2));
end 
fprintf( 'Tc   [%5.3f, %5.3f] \n', lb(nlevel+2,2), ub(nlevel+2,2));

[lb(:,3), ub(:,3)] = Kreider_example(L, X, Y3, Z, nlevel, ndraws);
disp(' ')
disp(['HRS Workers:  ratio = ', num2str(mean(Y3))])
for m = 1:nlevel+1
    fprintf( 'T%1.0f   [%5.3f, %5.3f] \n', m, lb(m,3), ub(m,3));
end 
fprintf( 'Tc   [%5.3f, %5.3f] \n', lb(nlevel+2,3), ub(nlevel+2,3));

[lb(:,4), ub(:,4)] = Kreider_example(L, X, Y4, Z, nlevel, ndraws);
disp(' ')
disp(['HRS Claim no disability:  ratio = ', num2str(mean(Y4))])
for m = 1:nlevel+1
    fprintf( 'T%1.0f   [%5.3f, %5.3f] \n', m, lb(m,4), ub(m,4));
end
fprintf( 'Tc   [%5.3f, %5.3f] \n', lb(nlevel+2,4), ub(nlevel+2,4));

[lb(:,5), ub(:,5)] = Kreider_example(L, X, Y5, Z, nlevel, ndraws);
disp(' ')
disp(['HRS All of the above:  ratio = ', num2str(mean(Y5))])
for m = 1:nlevel+1
    fprintf( 'T%1.0f   [%5.3f, %5.3f] \n', m, lb(m,5), ub(m,5));
end
fprintf( 'Tc   [%5.3f, %5.3f] \n', lb(nlevel+2,5), ub(nlevel+2,5));

        
% Write an EXCEL report
results = cell(nlevel+3, 6);
results(1,:) = {'', 'Beneficiaries', 'Wave 2', 'Workers', 'No disability', 'All of above'};
for m = 1:nlevel + 2
    if m < nlevel + 2
        results{m+1,1} = ['T', num2str(m)];
    else
        results{m+1,1} = 'Tc';
    end
    for n = 1:5
        results{m+1, n+1} = [ '['  num2str(lb(m,n),'%5.3f')  ', '  num2str(ub(m,n),'%5.3f')  ']' ];
    end
end

try
    disp('Writing an EXCEL report ...')
    xlswrite('Sim_results_Kreider.xls',results,'HRS','A1');    
end

%-------------------------------------------------
%-------------------------------------------------
% Loading SIPP data
data_raw = importdata('SIPP.txt');
data = [data_raw(1:3:end,:), data_raw(2:3:end,:), data_raw(3:3:end, 1:2)];
X = data(:, 3);
L = data(:, 4);
Y1 = data(:, 5);
Y2 = data(:, 11);
Y3 = L;
Y4 = 1 - X;
Y5 = (Y1 + Y2 + Y3 + Y4) >= 1;
Z = data(:,6) - 39;

% Compute the bootstap corrected MIV bounds under 5 verification scenarios.
lb = zeros(nlevel+2,5);
ub = zeros(nlevel+2,5);
[lb(:,1), ub(:,1)] = Kreider_example(L, X, Y1, Z, nlevel, ndraws);
disp(' ')
disp(['SIPP Beneficiaries:  ratio = ', num2str(mean(Y1))])
for m = 1:nlevel+1
    fprintf( 'T%1.0f   [%5.3f, %5.3f] \n', m, lb(m,1), ub(m,1));
end
fprintf( 'Tc   [%5.3f, %5.3f] \n', lb(nlevel+2,1), ub(nlevel+2,1));

[lb(:,2), ub(:,2)] = Kreider_example(L, X, Y2, Z, nlevel, ndraws);
disp(' ')
disp(['SIPP Wave 2 verification:  ratio = ', num2str(mean(Y2))])
for m = 1:nlevel+1
    fprintf( 'T%1.0f   [%5.3f, %5.3f] \n', m, lb(m,2), ub(m,2));
end
fprintf( 'Tc   [%5.3f, %5.3f] \n', lb(nlevel+2,2), ub(nlevel+2,2));

[lb(:,3), ub(:,3)] = Kreider_example(L, X, Y3, Z, nlevel, ndraws);
disp(' ')
disp(['SIPP Workers:  ratio = ', num2str(mean(Y3))])
for m = 1:nlevel+1
    fprintf( 'T%1.0f   [%5.3f, %5.3f] \n', m, lb(m,3), ub(m,3));
end 
fprintf( 'Tc   [%5.3f, %5.3f] \n', lb(nlevel+2,3), ub(nlevel+2,3));

[lb(:,4), ub(:,4)] = Kreider_example(L, X, Y4, Z, nlevel, ndraws);
disp(' ')
disp(['SIPP Claim no disability:  ratio = ', num2str(mean(Y4))])
for m = 1:nlevel+1
    fprintf( 'T%1.0f   [%5.3f, %5.3f] \n', m, lb(m,4), ub(m,4));
end
fprintf( 'Tc   [%5.3f, %5.3f] \n', lb(nlevel+2,4), ub(nlevel+2,4));

[lb(:,5), ub(:,5)] = Kreider_example(L, X, Y5, Z, nlevel, ndraws);
disp(' ')
disp(['SIPP All of the above:  ratio = ', num2str(mean(Y5))])
for m = 1:nlevel+1
    fprintf( 'T%1.0f   [%5.3f, %5.3f] \n', m, lb(m,5), ub(m,5));
end
fprintf( 'Tc   [%5.3f, %5.3f] \n', lb(nlevel+2,5), ub(nlevel+2,5));

% Write an EXCEL report
results = cell(nlevel+3, 6);
results(1,:) = {'Estimator', 'Beneficiaries', 'Wave 2', 'Workers', 'No disability', 'All of above'};
for m = 1:nlevel + 2
    if m < nlevel + 2
        results{m+1,1} = ['T', num2str(m)];
    else
        results{m+1,1} = 'Tc';
    end
    for n = 1:5
        results{m+1, n+1} = [ '['  num2str(lb(m,n),'%5.3f')  ', '  num2str(ub(m,n),'%5.3f')  ']' ];
    end
end

try
    disp('Writing an EXCEL report ...')
    xlswrite('Sim_results_Kreider.xls',results,'SIPP','A1');    
end

toc
         



