clear all
tic

% This routine replicate the simulation study in Manski and Pepper(2009) 
% Simultaneous multi-level bootstraps (Qian, 2011) added to further correct bias
% T1 raw estimator (max of the sample)
% T2 bootstrap corrected estimator
% T3 double bootstraps corrected estimator
% T4, T5,... higher level bootstrap corrected estimator
% This rountine calls the function Manski_example

myseed = 20110402;
try
    RandStream.setDefaultStream(RandStream('mt19937ar','seed',myseed));
catch
    randn('seed',myseed);
    rand('seed',myseed);
end

% Config parameters
nsim = 1000;
nlevel = 4;
ndraws = 100000;
var_flag = 1;

% Simulation study
specification = zeros(3,18);
bias = zeros(nlevel+1, 18);
bias_conserve = zeros(1,18);
bias_std = zeros(nlevel+1, 18);
bias_conserve_std = zeros(1,18);
k = 1;
for nobs = [100,500,1000]
    for M = [4,8]
        for sigma2 = [1,4,25]
            specification(:,k) = [nobs;M; sigma2];
            [bias(:,k),bias_conserve(k),bias_std(:,k),bias_conserve_std(k)] = Manski_example(M, nobs, sigma2, nsim, nlevel,ndraws, var_flag);
            k = k + 1;
        end
    end
end

% Write to EXCEL files
header =  cell(3+nlevel+1+1,1);
header{1} = 'n';
header{2} = 'M';
header{3} = 'S2';
for m = 1:nlevel+1
    header{m+3} = ['T',num2str(m)];
end
header{nlevel+5} = 'Tc';


try
    disp('Writing an EXCEL report ...')
    skip = nlevel + 6;
    xlswrite('Sim_results_Manski.xls',header,'Sheet2','A1');
    xlswrite('Sim_results_Manski.xls',specification,'Sheet2','B1');
    xlswrite('Sim_results_Manski.xls',[bias_std;bias_conserve_std],'Sheet2','B4');    
    
    xlswrite('Sim_results_Manski.xls',header,'Sheet1','A1');
    xlswrite('Sim_results_Manski.xls',specification(:,1:6),'Sheet1','B1');
    xlswrite('Sim_results_Manski.xls',[bias(:,1:6);bias_conserve(:,1:6)],'Sheet1','B4');
    
    xlswrite('Sim_results_Manski.xls',header,'Sheet1',['A',num2str(skip+1)]);
    xlswrite('Sim_results_Manski.xls',specification(:,7:12),'Sheet1',['B',num2str(skip+1)]);
    xlswrite('Sim_results_Manski.xls',[bias(:,7:12);bias_conserve(:,7:12)],'Sheet1',['B',num2str(skip+4)]);
    
    xlswrite('Sim_results_Manski.xls',header,'Sheet1',['A',num2str(skip*2+1)]);
    xlswrite('Sim_results_Manski.xls',specification(:,13:18),'Sheet1',['B',num2str(skip*2+1)]);
    xlswrite('Sim_results_Manski.xls',[bias(:,13:18);bias_conserve(:,13:18)],'Sheet1',['B',num2str(skip*2+4)]);
    
end

toc



