clear all
tic

% This routine is similar to the simulation study in Manski and Pepper(2009) 
% Simultaneous multi-level bootstraps (Qian, 2011) added to further correct bias
% The difference is the MIV has identification power here since we
% replace D = (Z + randn(nobs,1) ) > 0 with D = (Z + randn(nobs,1) ) < 0;
% So that the lower bound decrease with MIV.
% T1 raw estimator (max of the sample)
% T2 bootstrap corrected estimator
% T3 double bootstraps corrected estimator
% T4, T5,... higher level bootstrap corrected estimator
% This rountine calls the function Manski_example

myseed = 20110402;
try
    RandStream.setDefaultStream(RandStream('mt19937ar','seed',myseed));
catch
    randn('seed',myseed);
    rand('seed',myseed);
end

% Config parameters
M = 8;
nobs = 500;
sigma2 = 1;
nsim = 1000;
nlevel = 4;
ndraws = 10000;
var_flag = 1;


estimator = zeros(nsim, nlevel+1);
conserve = zeros(nsim,1);
H_status_bar = waitbar(0,'Simulation in Progress   0%');
percentage = floor(nsim / 100);
for r = 1:nsim
    
    if mod(r,percentage) == 0
        waitbar(r / nsim,H_status_bar,['Simulation in Progress   ',num2str(floor(r / nsim * 100)),'%']);
    end
    
    %Z = ceil(rand(nobs,1)*M) / M;
    Z = randi(M,[nobs,1]) / M;
    if length(unique(Z)) < M
        disp('MIV Estimator does not exist. Take another draw.')
        Z = randi(M,[nobs,1]) / M;
    end
    D = (Z + randn(nobs,1) ) < 0;
    Y = randn(nobs,1) * sqrt(sigma2);
    Y(Y>1.96) = 1.96;
    Y(Y<-1.96) = -1.96;
    
    if var_flag ~= 2
        Q = Y;
        Q(D==0) = -1.96;
        fL = zeros(1,M);
        fL_std = zeros(1,M);
        for m = 1:M
            ind = abs(Z - m/M) < 1e-8;
            fL(m) = mean(Q(ind));        
       
            prob_z = mean(ind);
            r_vec = 1:nobs;
            inv_size = sum(1./r_vec .* binopdf(r_vec,nobs,prob_z)) /(1-(1-prob_z)^nobs);
            %disp(['E(size) = ',num2str(1/inv_size),' Realized size = ',num2str(sum(ind))])
            fL_std(m) = sqrt( inv_size * var(Q(ind)) );
        end
    else
        ngrid = 1000;
        [fL, temp1, fL_std, temp2] = MIV_std(Y, D, Z, 1,ngrid);
    end

    estimator(r,:) = BIAS_CORRECT_MAX(fL, fL_std, nlevel, ndraws);
    conserve(r) = max(fL) - mean(max(randn(M,ndraws) .* repmat(fL_std',1,ndraws)));
end
close(H_status_bar);

disp(' ')
fprintf('n = %5.0f, M = %3.0f, ^2 = %3.0f,  \n', nobs, M, sigma2);
theory = -1.96 * normcdf(1/M);
bias = mean(estimator) - theory;
bias_std = std(estimator - theory) / sqrt(nsim);
bias_conserve = mean(conserve) - theory;
bias_conserve_std = std(conserve - theory) / sqrt(nsim);
disp(' ')
for m = 1:nlevel+1
    fprintf( 'Bias T%1.0f = %5.3f \n', m, bias(m));
end
fprintf( 'Bias Tc = %5.3f \n', bias_conserve);
    
toc



